@@ -1,6 +1,7 @@
 import urllib.request, json
 import logging
 import typing
+import datetime
 
 import jose.jwt
 from starlette.requests import Request
@@ -49,8 +50,12 @@ class AuthenticateMiddleware:
         * ``audience``: audience or the tokens
 
     If multiple providers are passed, the request is valid if any of the providers authenticates the request.
+
+    Use ``key_refresh_minutes`` to change the frequency on which the key is retrieved from the metadata.
+    Defaults to `None`, in which case the key is only retrieved once. If a dictionary is provided, it is assumed to be
+    have the same keys as ``providers`` and corresponds to different refresh times for each provider.
     """
-    def __init__(self, app: ASGIApp, providers, public_paths=None, get_keys=None) -> None:
+    def __init__(self, app: ASGIApp, providers, public_paths=None, get_keys=None, key_refresh_minutes=None) -> None:
         self._app = app
         for provider in providers:
             _validate_provider(provider, providers[provider])
@@ -58,7 +63,15 @@ class AuthenticateMiddleware:
         self._get_keys = get_keys or _get_keys
         self._public_paths = public_paths or set()
 
-        # cached attribute
+        if key_refresh_minutes is None:
+            self._timeout = {provider: None for provider in providers}
+        elif isinstance(key_refresh_minutes, dict):
+            self._timeout = {provider: datetime.timedelta(minutes=key_refresh_minutes[provider]) for provider in providers}
+        else:
+            self._timeout = {provider: datetime.timedelta(minutes=key_refresh_minutes) for provider in providers}
+
+        # cached attribute and respective timeout
+        self._last_retrieval = {}
         self._keys = {}
 
     def _provider_claims(self, provider, token):
@@ -129,10 +142,28 @@ class AuthenticateMiddleware:
 
         return await self._app(scope, receive, send)
 
+    def _should_refresh(self, provider: str):
+        if self._keys.get(provider, None) is None:
+            # we do not even have the key (first time) => should refresh
+            return True
+        elif self._timeout[provider] is None:
+            # we have a key and no timeout => do not refresh
+            return False
+        # have the key and have timeout => check if we passed the timeout
+        return self._last_retrieval[provider] + self._timeout[provider] < datetime.datetime.utcnow()
+
+    def _refresh_keys(self, provider: str):
+        """
+        Refreshes the keys by calling `get_keys`.
+        This should be called sparsly, as it typically results in a blocking https request.
+        """
+        self._keys[provider] = self._get_keys(self._providers[provider]['keys'])
+        self._last_retrieval[provider] = datetime.datetime.utcnow()
+
     def _provider_keys(self, provider: str):
         """
         Returns the jw keys of the provider. This is retrieved from the internet and cached at the first retrieval.
         """
-        if self._keys.get(provider, None) is None:
-            self._keys[provider] = self._get_keys(self._providers[provider]['keys'])
+        if self._should_refresh(provider):
+            self._refresh_keys(provider)
         return self._keys[provider]
